#!/bin/bash
# This is a script to build a Zenwalk package. Check the Zenwalk homepage: www.zenwalk.org
#
# SOFTWARE			: wixi
# PROJECT HOMEPAGE 	: http://wixi.sourceforge.net
# SOURCES          	: http://downloads.sourceforge.net/wixi/
#
# AUTHOR         	: skywatcher (s.skywatcher|AT|gmail|DOT|com, simone antoniucci)
# LATEST REVISION   : 2009-12-03 
#
# BUILD-ONLY DEPS	: -
# NOTES				: -

### Setting main variables
NAME="wixi"
VERSION="1.10"
ARCH="i686"
CPU="i686"
BUILD=64.1
EXT="txz"

HOMEPAGE="http://wixi.sourceforge.net"
DESCRIPTION="Desktop wiki application"
SRCLNK="http://downloads.sourceforge.net/wixi/$NAME-$VERSION.zip"

CWD=`pwd`
PKGDIR=$CWD/$NAME-install
PKG=$NAME-$VERSION-$ARCH-$BUILD
SRCPKG=`echo $SRCLNK | rev | cut -d/ -f1 | rev`
SRCDIR=$CWD/$NAME-$VERSION
ICNDIR=$PKGDIR/usr/share/icons/hicolor/48x48/apps/		

URL=http://zenwalk.pinguix.com/user-accounts/skywatcher/$NAME/$VERSION
DOCS="Changelog COPYING INSTALL README doc/t2t doc/*.html"
ICN=$NAME.png
ICNLNK=$URL/$ICN

NJOBS="-j2"
export CFLAGS="-O2 -march=$ARCH -mtune=$CPU"
export CXXFLAGS="-O2 -march=$ARCH -mtune=$CPU"

### Cleaning
rm -rf $PKGDIR
rm -rf $SRCDIR

### Mkdir
mkdir -p $PKGDIR
mkdir -p $PKGDIR/install
mkdir -p $PKGDIR/usr/doc/$NAME-$VERSION
mkdir -p $PKGDIR/usr/src/$NAME-$VERSION
mkdir -p $PKGDIR/usr/share/applications
mkdir -p $ICNDIR

### Checking if source is present. If it's not, download it
if [ -f $SRCPKG ]; then
	echo "Source file is already present, no need to download it";
else
 	echo "Downloading source..."
	wget -c -nv $SRCLNK
fi

### Extracting source
unzip $SRCPKG

### Configure, make, install
cd $SRCDIR
python setup.py install --prefix=/usr --root=$PKGDIR

### Stripping binaries
cd $PKGDIR
find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null

### Docs, man, info
cd $SRCDIR
cp -a $DOCS $PKGDIR/usr/doc/$NAME-$VERSION
#find $PKGDIR/usr/man -type f -exec gzip -9 {} \;

### Adding build-file and other stuff to the package
cp -a $CWD/$0 $PKGDIR/usr/src/$NAME-$VERSION/build-$NAME.sh
chmod a+x $PKGDIR/usr/src/$NAME-$VERSION/build-$NAME.sh
cp -a $CWD/$ICN $PKGDIR/usr/src/$NAME-$VERSION

### Slack-desc 
#    |-----handy-ruler------------------------------------------------------|
cat > $PKGDIR/install/slack-desc << EOF
$NAME: $NAME - $DESCRIPTION
$NAME:  
$NAME: Wixi is a multi-platform wiki application for the desktop. It is 
$NAME: written in python/wxpython and uses txt2tags to generate plain text
$NAME: to many other formats. The goal is to provide a simple wiki tool 
$NAME: around the txt2tags document generator and also be multi-platform.
$NAME: 
$NAME: 
$NAME: 
$NAME: 
$NAME: $HOMEPAGE
EOF

### Desktop entry
cat > $PKGDIR/usr/share/applications/$NAME.desktop << EOF
[Desktop Entry]
Name=wixi
Comment=$DESCRIPTION
Exec=wixi
Icon=wixi
Type=Application
Categories=Application;Utility;
Terminal=false
StartupNotify=true
Encoding=UTF-8
EOF

### Icon
cp -a $CWD/$ICN $ICNDIR

### Ownerships and permissions
cd $PKGDIR
chown -R root:root .
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;
find . -perm 600 -exec chmod 644 {} \;
find . -perm 444 -exec chmod 644 {} \;
find . -perm 400 -exec chmod 644 {} \;
find . -perm 440 -exec chmod 644 {} \;
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 511 -exec chmod 755 {} \;
find . -perm 711 -exec chmod 755 {} \;
find . -perm 555 -exec chmod 755 {} \;

### Making the package
makepkg -l y -c n $CWD/$PKG.$EXT

### md5
cd $CWD
md5sum $PKG.$EXT > $PKG.md5

### dep
ADD="python,wxpython" depfinder $PKG.$EXT > $PKG.dep

### txt
cat $PKGDIR/install/slack-desc > $PKG.txt

### src
rm -fr $PKG.src
for link in $URL/$0 $SRCLNK $ICNLNK
	do echo $link >> $PKG.src
done

### Cleaning
rm -rf $SRCDIR
rm -rf $PKGDIR

### Done. Ciao!
